@@ -1,6 +1,6 @@
-/* mpn_mod_1s_3p (ap, n, b, cps)
+/* mpn_mod_1s_2p (ap, n, b, cps)
    Divide (ap,,n) by b.  Return the single-limb remainder.
-   Requires that d < B / 3.
+   Requires that b < B / 2.
 
    Contributed to the GNU project by Torbjorn Granlund.
 
@@ -30,13 +30,13 @@ along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 #include "longlong.h"
 
 void
-mpn_mod_1s_3p_cps (mp_limb_t cps[6], mp_limb_t b)
+mpn_mod_1s_2p_cps (mp_limb_t cps[5], mp_limb_t b)
 {
   mp_limb_t bi;
-  mp_limb_t B1modb, B2modb, B3modb, B4modb;
+  mp_limb_t B1modb, B2modb, B3modb;
   int cnt;
 
-  ASSERT (b <= (~(mp_limb_t) 0) / 3);
+  ASSERT (b <= (~(mp_limb_t) 0) / 2);
 
   count_leading_zeros (cnt, b);
 
@@ -47,20 +47,18 @@ mpn_mod_1s_3p_cps (mp_limb_t cps[6], mp_limb_t b)
   ASSERT (B1modb <= b);		/* NB: not fully reduced mod b */
   udiv_rnd_preinv (B2modb, B1modb, b, bi);
   udiv_rnd_preinv (B3modb, B2modb, b, bi);
-  udiv_rnd_preinv (B4modb, B3modb, b, bi);
 
   cps[0] = bi;
   cps[1] = cnt;
   cps[2] = B1modb >> cnt;
   cps[3] = B2modb >> cnt;
   cps[4] = B3modb >> cnt;
-  cps[5] = B4modb >> cnt;
 
 #if WANT_ASSERT
   {
     int i;
     b = cps[2];
-    for (i = 3; i <= 5; i++)
+    for (i = 3; i <= 4; i++)
       {
 	b += cps[i];
 	ASSERT (b >= cps[i]);
@@ -70,10 +68,10 @@ mpn_mod_1s_3p_cps (mp_limb_t cps[6], mp_limb_t b)
 }
 
 mp_limb_t
-mpn_mod_1s_3p (mp_srcptr ap, mp_size_t n, mp_limb_t b, mp_limb_t cps[6])
+mpn_mod_1s_2p (mp_srcptr ap, mp_size_t n, mp_limb_t b, mp_limb_t cps[5])
 {
   mp_limb_t rh, rl, bi, q, ph, pl, ch, cl, r;
-  mp_limb_t B1modb, B2modb, B3modb, B4modb;
+  mp_limb_t B1modb, B2modb, B3modb;
   mp_size_t i;
   int cnt;
 
@@ -82,49 +80,45 @@ mpn_mod_1s_3p (mp_srcptr ap, mp_size_t n, mp_limb_t b, mp_limb_t cps[6])
   B1modb = cps[2];
   B2modb = cps[3];
   B3modb = cps[4];
-  B4modb = cps[5];
 
-  /* We compute n mod 3 in a tricky way, which works except for when n is so
-     close to the maximum size that we don't need to support it.  */
-  switch ((mp_limb_t) n * MODLIMB_INVERSE_3 >> (GMP_NUMB_BITS - 2))
+  if ((n & 1) != 0)
     {
-    case 0:
+      if (n == 1)
+	{
+	  rl = ap[n - 1];
+	  bi = cps[0];
+	  cnt = cps[1];
+	  udiv_qrnnd_preinv (q, r, rl >> (GMP_LIMB_BITS - cnt),
+			     rl << cnt, b, bi);
+	  return r >> cnt;
+	}
+
       umul_ppmm (ph, pl, ap[n - 2], B1modb);
       add_ssaaaa (ph, pl, ph, pl, 0, ap[n - 3]);
       umul_ppmm (rh, rl, ap[n - 1], B2modb);
       add_ssaaaa (rh, rl, rh, rl, ph, pl);
-      n -= 3;
-      break;
-    case 2:	/* n mod 3 = 1 */
-      rh = 0;
-      rl = ap[n - 1];
-      n -= 1;
-      break;
-    case 1:	/* n mod 3 = 2 */
-      umul_ppmm (ph, pl, ap[n - 1], B1modb);
-      add_ssaaaa (rh, rl, ph, pl, 0, ap[n - 2]);
-      n -= 2;
-      break;
+      n--;
+    }
+  else
+    {
+      umul_ppmm (rh, rl, ap[n - 1], B1modb);
+      add_ssaaaa (rh, rl, rh, rl, 0, ap[n - 2]);
     }
 
-  for (i = n - 3; i >= 0; i -= 3)
+  for (i = n - 4; i >= 0; i -= 2)
     {
       /* rr = ap[i]				< B
 	    + ap[i+1] * (B mod b)		<= (B-1)(b-1)
-	    + ap[i+2] * (B^2 mod b)		<= (B-1)(b-1)
-	    + LO(rr)  * (B^3 mod b)		<= (B-1)(b-1)
-	    + HI(rr)  * (B^4 mod b)		<= (B-1)(b-1)
+	    + LO(rr)  * (B^2 mod b)		<= (B-1)(b-1)
+	    + HI(rr)  * (B^3 mod b)		<= (B-1)(b-1)
       */
       umul_ppmm (ph, pl, ap[i + 1], B1modb);
       add_ssaaaa (ph, pl, ph, pl, 0, ap[i + 0]);
 
-      umul_ppmm (ch, cl, ap[i + 2], B2modb);
+      umul_ppmm (ch, cl, rl, B2modb);
       add_ssaaaa (ph, pl, ph, pl, ch, cl);
 
-      umul_ppmm (ch, cl, rl, B3modb);
-      add_ssaaaa (ph, pl, ph, pl, ch, cl);
-
-      umul_ppmm (rh, rl, rh, B4modb);
+      umul_ppmm (rh, rl, rh, B3modb);
       add_ssaaaa (rh, rl, rh, rl, ph, pl);
     }
 
@@ -138,7 +132,7 @@ mpn_mod_1s_3p (mp_srcptr ap, mp_size_t n, mp_limb_t b, mp_limb_t cps[6])
 #else
   udiv_qrnnd_preinv (q, r, rh >> (GMP_LIMB_BITS - cnt),
 		     (rh << cnt) | (rl >> (GMP_LIMB_BITS - cnt)), b, bi);
-  ASSERT (q <= 3);	/* optimize for small quotient? */
+  ASSERT (q <= 2);	/* optimize for small quotient? */
 #endif
 
   udiv_qrnnd_preinv (q, r, r, rl << cnt, b, bi);
