@@ -17,13 +17,17 @@ License for more details.
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 
+#define __gmpn_set_str mpn_set_str_subquad
+#define __gmpn_bc_set_str mpn_bc_set_str_subquad
+#define __gmpn_dc_set_str mpn_dc_set_str_subquad
+#define __gmpn_set_str_compute_powtab mpn_set_str_compute_powtab_subquad
+
 #include "gmp.h"
 #include "gmp-impl.h"
 
-/* Note this is designed only for SET_STR_BLOCK_SIZE==1. */
-
-#undef SET_STR_THRESHOLD
-#define SET_STR_THRESHOLD  2 /* never */
-#define __gmpn_set_str mpn_set_str_subquad
+#undef SET_STR_DC_THRESHOLD
+#define SET_STR_DC_THRESHOLD  2 /* never */
+#undef SET_STR_PRECOMPUTE_THRESHOLD
+#define SET_STR_PRECOMPUTE_THRESHOLD  2 /* never */
 
 #include "mpn/generic/set_str.c"
